package samples.extraction.pricing;

import java.math.BigDecimal;
import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ExtractionUtility;
import samples.ServerErrorException;
import samples.JavaUtility;
import core.ISample;

public class PremiumPricingSample implements ISample {

	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("CUSIP");
		outputFields.getString().add("Issuer Name");
		outputFields.getString().add("Maturity Date");
		outputFields.getString().add("Previous Close Price");
		outputFields.getString().add("Bid Price");
		outputFields.getString().add("Ask Price");
		outputFields.getString().add("Mid Price");

		System.out
				.println("Performing a 3PM US Premium Pricing extraction that:");
		System.out.println("- gets its instruments from a Gov/Corp search");
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		String subGroup = "GBND";
		String spType = "AAA";
		String country = "US";
		String assetStatusType = "ISS";
		String contributorType = "EJV";
		int couponLimit = 1;
		int matureYears = 2;
		ArrayOfString splist = new ArrayOfString();
		splist.getString().add(spType);
		ArrayOfString assetlist = new ArrayOfString();
		assetlist.getString().add(assetStatusType);

		System.out.println("Searching for Gov/Corp Instruments that:");
		System.out.println("- are government bonds");
		System.out.println("- have a sub-group type of 'Notes/Bonds Govt' ("
				+ subGroup + ")");
		System.out.println("- have a S&P rating of {" + spType + ")");
		System.out.println("- are from the United States (" + country + ")");
		System.out.println("- are issued (" + assetStatusType + ")");
		System.out.println("- have a contributor of Tr Pricing Service ("
				+ contributorType + ")");
		System.out.println("- have a coupon value that is greater than "
				+ couponLimit);
		System.out.println("- mature in more than " + matureYears + " years");

		InstrumentSearchRequestGovCorp searchRequest = new InstrumentSearchRequestGovCorp();
		searchRequest.setIdentifierType("RIC");
		searchRequest.setIdentifierSearchValue("*");
		searchRequest.setPreferredIdentifierType("RIC");
		searchRequest.setIsGovernmentGroup(true);
		searchRequest.setSubGroupType(subGroup);
		searchRequest.setStandardPoorsTypes(splist);
		searchRequest.setCountryType(country);
		searchRequest.setAssetStatusTypes(assetlist);
		searchRequest.setContributorType(contributorType);
		searchRequest.setCouponComparisonOperator(">");
		searchRequest.setCouponPrimaryValue(new BigDecimal(couponLimit));
		searchRequest.setMaturityDateComparisonOperator(">");
		searchRequest.setMaturityDatePrimaryValue(JavaUtility
				.asXMLGregorianCalendar(JavaUtility
						.dateadd("YEAR", matureYears)));

		InstrumentSearchResponse searchresponse = new InstrumentSearchResponse();
		try {
			searchresponse = proxy.searchInstruments(searchRequest, hch, null,
					hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println(Integer.toString(searchresponse.getInstruments()
				.getValidatedSearchInstrument().size())
				+ " instruments found:");
		for (int n = 0; n < searchresponse.getInstruments()
				.getValidatedSearchInstrument().size(); n++) {
			System.out.println("  "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n)
							.getIdentifierType()
					+ ": "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n)
							.getIdentifierValue()
					+ " "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n).getSource());
		}

		System.out.println("Performing extraction...");
		InstrumentExtractionRequestPremiumPricingUS3PM request = new InstrumentExtractionRequestPremiumPricingUS3PM();
		ArrayOfValidatedInstrument arrayValidated = ExtractionUtility
				.ChangeDataType(searchresponse.getInstruments());

		request.setOutputFields(outputFields);
		request.setInstruments(arrayValidated);

		ExtractionResponse response = new ExtractionResponse();
		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out.println();
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
		System.out.println();

		for (int i = 0; i < response.getColumns().getExtractionColumn().size(); i++) {
			System.out.print(response.getColumns().getExtractionColumn().get(i)
					.getName()
					+ ",");
		}
		System.out.println();

		for (int j = 0; j < response.getRowCount(); j++) {
			for (int i = 0; i < response.getColumns().getExtractionColumn()
					.size(); i++) {
				if (response.getColumns().getExtractionColumn().get(i)
						.getValues().getDoubleOrEmptyOrDate().get(j) instanceof EmptyValue) {
					System.out.print(",");
				} else {
					System.out.print(response.getColumns()
							.getExtractionColumn().get(i).getValues()
							.getDoubleOrEmptyOrDate().get(j).toString()
							+ ",");
				}
			}
			System.out.println();
		}
		ExtractionUtility.EnsureOutputFields(response);

		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");
	}
}
